/*                                    _
 *   ___ _ __ ___ _ __ ___   ___   __| |
 *  / __| '__/ __| '_ ` _ \ / _ \ / _` |
 *  \__ \ |  \__ \ | | | | | (_) | (_| |
 *  |___/_|  |___/_| |_| |_|\___/ \__,_|                              
 *
 *  __|_ _  _  _| _  _ _|._  _  _|   _ _    _  _ _  _    __|_ _   __|_   _ _ 
 * _\ | (_|| |(_|(_|| (_||/_(/_(_|  | (/_\/(_|| | ||_)  _\ | ||_|(_ ||_|| (/_
 *                                                 |    
 * 	Find the latest everything at: 
 *		http://srsmod.googlecode.com/
 *
 */
 
#pragma semicolon 1
#pragma tabsize 4
 
#include <sourcemod>
#include <sdktools>
#include <left4downtown>
#include <sdkhooks>

#define SRS_VERSION 							"1.9 DEV"
#define SRS_DEBUG 								0
#define SRS_DEBUG_LOG 							1
#define SRS_DEBUG_SPAM 							0

#define SRS_VERSION_REQUIRED_SOURCEMOD 			"1.3.4"
#define SRS_VERSION_REQUIRED_SOURCEMOD_NONDEV 	0 //1 dont allow -dev version, 0 ignore -dev version
#define SRS_VERSION_REQUIRED_LEFT4DOWNTOWN 		"0.5.2.3"

#include "srs.helpers.sp"
#include "srs.statehelpers.sp"

#include "srs.unprohibitbosses.sp"
#include "srs.despawninfected.sp"
#include "srs.waterbrake.sp"
#include "srs.itemremoval.sp"
#include "srs.tankprelight.sp"
#include "srs.finaleghostspawnrange.sp"
#include "srs.scoringsystem.sp"
#include "srs.tankmisspawn.sp"
#include "srs.tankpunchrock.sp"

public Plugin:myinfo = 
{
	name = "Standardized Revamp Structure Modification for L4D2",
	author = "srsteam",
	description = "Changes the gameplay to be more challenging and even",
	version = SRS_VERSION,
	url = "http://srsmod.googlecode.com/"
}

public OnPluginStart()
{
	StateHelpers_OnPluginStart();
	LoadTranslations("common.phrases");
	LoadTranslations("plugin.srsmod");

	Versioning_CheckDependencies();

	UnprohibitBosses_OnModuleLoaded();
	DespawnInfected_OnModuleLoaded();
	WaterBrake_OnModuleLoaded();
	ItemRemoval_OnModuleLoaded();
	TankPrelight_OnModuleLoaded();
	FinaleRangeMod_OnModuleLoaded();
	Scoring_OnModuleLoaded();
	TankMisspawn_OnModuleLoaded();
	TankPunchRock_OnModuleLoaded();
	
	CreateConVar("srs_version", SRS_VERSION, "srsmod version.", SRS_CVAR_DEFAULT_FLAGS | FCVAR_DONTRECORD);
}

public OnPluginEnd()
{
	StateHelpers_OnPluginEnd();
}

public Action:OnPlayerRunCmd(client, &buttons, &impulse, Float:vel[3], Float:angles[3], &weapon)
{
	TankPrelight_OnPlayerRunCmd(client, buttons);
	return Plugin_Continue;
}

public OnClientPostAdminCheck(client)
{
	_FR_OnClientPostAdminCheck(client);
	SDKHook(client, SDKHook_OnTakeDamage, OnTakeDamage);
	SDKHook(client, SDKHook_WeaponEquip, WeaponEquip);
}

public OnEntityCreated(entity, const String:classname[])
{
	_DI_OnEntityCreated(entity, classname);
	_IR_OnEntityCreated(entity, classname);
}

public OnEntityDestroyed(entity)
{
	_DI_OnEntityDestroyed(entity);
}

public Action:OnTakeDamage(victim, &attacker, &inflictor, &Float:damage, &damagetype)
{
	_SC_OnTakeDamage(victim, attacker, inflictor, damage);
	return Plugin_Continue;
}

public Action:WeaponEquip(client, entity)
{
	_IR_WeaponEquip(entity);
	return Plugin_Continue;
}

public OnGameFrame()
{
	WaterBrake_OnGameFrame();
}

public OnMapStart()
{
	StateHelpers_OnMapStart();
	ItemRemoval_OnMapStart();
}

public OnMapEnd()
{
	StateHelpers_OnMapEnd();
}